@@ -1,31 +1,28 @@
 import copy
 import os
-import traceback
 
 from PyQt5.QtCore import QDir, Qt, pyqtSlot, QTimer
 from PyQt5.QtGui import QIcon, QCloseEvent, QKeySequence
-from PyQt5.QtWidgets import QMainWindow, QUndoGroup, QActionGroup, QHeaderView, QAction, QFileDialog, \
-    QMessageBox, QApplication, qApp
+from PyQt5.QtWidgets import QMainWindow, QUndoGroup, QActionGroup, QHeaderView, QAction, QMessageBox, QApplication, qApp
 
-from urh import constants, version
-from urh.controller.dialogs.CSVImportDialog import CSVImportDialog
+from urh import settings, version
 from urh.controller.CompareFrameController import CompareFrameController
-from urh.controller.dialogs.DecoderDialog import DecoderDialog
 from urh.controller.GeneratorTabController import GeneratorTabController
+from urh.controller.SignalTabController import SignalTabController
+from urh.controller.SimulatorTabController import SimulatorTabController
+from urh.controller.dialogs.CSVImportDialog import CSVImportDialog
+from urh.controller.dialogs.DecoderDialog import DecoderDialog
 from urh.controller.dialogs.OptionsDialog import OptionsDialog
 from urh.controller.dialogs.ProjectDialog import ProjectDialog
 from urh.controller.dialogs.ProtocolSniffDialog import ProtocolSniffDialog
 from urh.controller.dialogs.ReceiveDialog import ReceiveDialog
-from urh.controller.widgets.SignalFrame import SignalFrame
-from urh.controller.SignalTabController import SignalTabController
-from urh.controller.SimulatorTabController import SimulatorTabController
 from urh.controller.dialogs.SpectrumDialogController import SpectrumDialogController
+from urh.controller.widgets.SignalFrame import SignalFrame
 from urh.models.FileFilterProxyModel import FileFilterProxyModel
 from urh.models.FileIconProvider import FileIconProvider
 from urh.models.FileSystemModel import FileSystemModel
 from urh.models.ParticipantLegendListModel import ParticipantLegendListModel
 from urh.plugins.PluginManager import PluginManager
-from urh.signalprocessing.Message import Message
 from urh.signalprocessing.ProtocolAnalyzer import ProtocolAnalyzer
 from urh.signalprocessing.Signal import Signal
 from urh.ui.ui_main import Ui_MainWindow
@@ -82,14 +79,13 @@ class MainController(QMainWindow):
         self.cancel_action.setIcon(QIcon.fromTheme("dialog-cancel"))
         self.addAction(self.cancel_action)
 
-        self.ui.actionAuto_detect_new_signals.setChecked(constants.SETTINGS.value("auto_detect_new_signals",
-                                                                                  True, bool))
+        self.ui.actionAuto_detect_new_signals.setChecked(settings.read("auto_detect_new_signals", True, bool))
 
         self.participant_legend_model = ParticipantLegendListModel(self.project_manager.participants)
         self.ui.listViewParticipants.setModel(self.participant_legend_model)
 
         gtc = self.generator_tab_controller
-        gtc.ui.splitter.setSizes([gtc.width() / 0.7, gtc.width() / 0.3])
+        gtc.ui.splitter.setSizes([int(gtc.width() / 0.7), int(gtc.width() / 0.3)])
 
         self.ui.tab_generator.layout().addWidget(self.generator_tab_controller)
 
@@ -103,9 +99,22 @@ class MainController(QMainWindow):
         self.ui.actionNone.setActionGroup(group)
         self.ui.actionPSK.setActionGroup(group)
 
+        noise_threshold_setting = settings.read("default_noise_threshold", "automatic")
+        noise_threshold_group = QActionGroup(self)
+        self.ui.actionAutomaticNoiseThreshold.setActionGroup(noise_threshold_group)
+        self.ui.actionAutomaticNoiseThreshold.setChecked(noise_threshold_setting == "automatic")
+        self.ui.action1NoiseThreshold.setActionGroup(noise_threshold_group)
+        self.ui.action1NoiseThreshold.setChecked(noise_threshold_setting == "1")
+        self.ui.action5NoiseThreshold.setActionGroup(noise_threshold_group)
+        self.ui.action5NoiseThreshold.setChecked(noise_threshold_setting == "5")
+        self.ui.action10NoiseThreshold.setActionGroup(noise_threshold_group)
+        self.ui.action10NoiseThreshold.setChecked(noise_threshold_setting == "10")
+        self.ui.action100NoiseThreshold.setActionGroup(noise_threshold_group)
+        self.ui.action100NoiseThreshold.setChecked(noise_threshold_setting == "100")
+
         self.recentFileActionList = []
         self.create_connects()
-        self.init_recent_file_action_list(constants.SETTINGS.value("recentFiles", []))
+        self.init_recent_file_action_list(settings.read("recentFiles", [], list))
 
         self.filemodel = FileSystemModel(self)
         path = QDir.homePath()
@@ -144,15 +153,17 @@ class MainController(QMainWindow):
         self.ui.splitter.setSizes([0, 1])
         self.refresh_main_menu()
 
-        self.apply_default_view(constants.SETTINGS.value('default_view', type=int))
+        self.apply_default_view(settings.read('default_view', type=int))
         self.project_save_timer.start(ProjectManager.AUTOSAVE_INTERVAL_MINUTES * 60 * 1000)
 
         self.ui.actionProject_settings.setVisible(False)
         self.ui.actionSave_project.setVisible(False)
         self.ui.actionClose_project.setVisible(False)
 
+        self.restoreGeometry(settings.read("{}/geometry".format(self.__class__.__name__), type=bytes))
+
     def __set_non_project_warning_visibility(self):
-        show = constants.SETTINGS.value("show_non_project_warning", True, bool) and not self.project_manager.project_loaded
+        show = settings.read("show_non_project_warning", True, bool) and not self.project_manager.project_loaded
         self.ui.labelNonProjectMode.setVisible(show)
 
     def create_connects(self):
@@ -184,6 +195,12 @@ class MainController(QMainWindow):
         self.ui.actionSamples_from_csv.triggered.connect(self.on_import_samples_from_csv_action_triggered)
         self.ui.actionAuto_detect_new_signals.triggered.connect(self.on_auto_detect_new_signals_action_triggered)
 
+        self.ui.actionAutomaticNoiseThreshold.triggered.connect(self.on_action_automatic_noise_threshold_triggered)
+        self.ui.action1NoiseThreshold.triggered.connect(self.on_action_1_noise_threshold_triggered)
+        self.ui.action5NoiseThreshold.triggered.connect(self.on_action_5_noise_threshold_triggered)
+        self.ui.action10NoiseThreshold.triggered.connect(self.on_action_10_noise_threshold_triggered)
+        self.ui.action100NoiseThreshold.triggered.connect(self.on_action_100_noise_threshold_triggered)
+
         self.ui.btnFileTreeGoUp.clicked.connect(self.on_btn_file_tree_go_up_clicked)
         self.ui.fileTree.directory_open_wanted.connect(self.project_manager.set_project_folder)
 
@@ -230,7 +247,7 @@ class MainController(QMainWindow):
         self.ui.labelNonProjectMode.linkActivated.connect(self.on_label_non_project_mode_link_activated)
 
         self.ui.menuFile.addSeparator()
-        for i in range(constants.MAX_RECENT_FILE_NR):
+        for i in range(settings.MAX_RECENT_FILE_NR):
             recent_file_action = QAction(self)
             recent_file_action.setVisible(False)
             recent_file_action.triggered.connect(self.on_open_recent_action_triggered)
@@ -301,14 +318,20 @@ class MainController(QMainWindow):
         if self.ui.actionAuto_detect_new_signals.isChecked() and not has_entry and not signal.changed:
             sig_frame.ui.stackedWidget.setCurrentWidget(sig_frame.ui.pageLoading)
             qApp.processEvents()
-            signal.auto_detect(detect_modulation=True, detect_noise=False)
+            if not signal.already_demodulated:
+                signal.auto_detect(detect_modulation=True, detect_noise=False)
             sig_frame.ui.stackedWidget.setCurrentWidget(sig_frame.ui.pageSignal)
 
         signal.blockSignals(False)
 
         self.signal_protocol_dict[sig_frame] = pa
 
-        sig_frame.refresh(draw_full_signal=True)  # protocol is derived here
+        sig_frame.refresh_signal(draw_full_signal=True)
+        sig_frame.refresh_signal_information(block=True)
+
+        qApp.processEvents()
+        sig_frame.show_protocol(refresh=True)
+
         if self.project_manager.read_participants_for_signal(signal, pa.messages):
             sig_frame.ui.gvSignal.redraw_view()
 
@@ -398,7 +421,7 @@ class MainController(QMainWindow):
             elif filename.endswith(".csv"):
                 self.__import_csv(filename, group_id)
                 continue
-            elif os.path.basename(filename) == constants.PROJECT_FILE:
+            elif os.path.basename(filename) == settings.PROJECT_FILE:
                 self.project_manager.set_project_folder(os.path.split(filename)[0])
             else:
                 self.add_signalfile(filename, group_id, enforce_sample_rate=enforce_sample_rate)
@@ -413,6 +436,7 @@ class MainController(QMainWindow):
 
     def closeEvent(self, event: QCloseEvent):
         self.save_project()
+        settings.write("{}/geometry".format(self.__class__.__name__), self.saveGeometry())
         super().closeEvent(event)
 
     def close_all_files(self):
@@ -508,16 +532,15 @@ class MainController(QMainWindow):
         if file_path in FileOperator.archives.keys():
             file_path = copy.copy(FileOperator.archives[file_path])
 
-        settings = constants.SETTINGS
-        recent_file_paths = settings.value("recentFiles", [])
+        recent_file_paths = settings.read("recentFiles", [], list)
         recent_file_paths = [] if recent_file_paths is None else recent_file_paths  # check None for OSX
         recent_file_paths = [p for p in recent_file_paths if p != file_path and p is not None and os.path.exists(p)]
         recent_file_paths.insert(0, file_path)
-        recent_file_paths = recent_file_paths[:constants.MAX_RECENT_FILE_NR]
+        recent_file_paths = recent_file_paths[:settings.MAX_RECENT_FILE_NR]
 
         self.init_recent_file_action_list(recent_file_paths)
 
-        settings.setValue("recentFiles", recent_file_paths)
+        settings.write("recentFiles", recent_file_paths)
 
     def init_recent_file_action_list(self, recent_file_paths: list):
         for i in range(len(self.recentFileActionList)):
@@ -728,7 +751,6 @@ class MainController(QMainWindow):
 
     @pyqtSlot()
     def on_new_project_action_triggered(self):
-        self.close_project()
         pdc = ProjectDialog(parent=self)
         pdc.finished.connect(self.on_project_dialog_finished)
         pdc.show()
@@ -754,7 +776,11 @@ class MainController(QMainWindow):
     @pyqtSlot()
     def on_project_dialog_finished(self):
         if self.sender().committed:
-            self.project_manager.from_dialog(self.sender())
+            if self.sender().new_project:
+                self.close_project()
+                self.project_manager.from_dialog(self.sender())
+            else:
+                self.project_manager.project_updated.emit()
 
     @pyqtSlot()
     def on_open_file_action_triggered(self):
@@ -877,7 +903,7 @@ class MainController(QMainWindow):
 
     @pyqtSlot(bool)
     def on_auto_detect_new_signals_action_triggered(self, checked: bool):
-        constants.SETTINGS.setValue("auto_detect_new_signals", bool(checked))
+        settings.write("auto_detect_new_signals", bool(checked))
 
     def __import_csv(self, file_name, group_id=0):
         def on_data_imported(complex_file, sample_rate):
@@ -892,7 +918,7 @@ class MainController(QMainWindow):
     def on_label_non_project_mode_link_activated(self, link: str):
         if link == "dont_show_non_project_again":
             self.ui.labelNonProjectMode.hide()
-            constants.SETTINGS.setValue("show_non_project_warning", False)
+            settings.write("show_non_project_warning", False)
         elif link == "open_new_project_dialog":
             self.on_new_project_action_triggered()
 
@@ -919,3 +945,23 @@ class MainController(QMainWindow):
         self.ui.tabWidget.setCurrentIndex(1)
         self.compare_frame_controller.add_protocol(protocol)
         self.compare_frame_controller.refresh()
+
+    @pyqtSlot()
+    def on_action_automatic_noise_threshold_triggered(self):
+        settings.write("default_noise_threshold", "automatic")
+
+    @pyqtSlot()
+    def on_action_1_noise_threshold_triggered(self):
+        settings.write("default_noise_threshold", "1")
+
+    @pyqtSlot()
+    def on_action_5_noise_threshold_triggered(self):
+        settings.write("default_noise_threshold", "5")
+
+    @pyqtSlot()
+    def on_action_10_noise_threshold_triggered(self):
+        settings.write("default_noise_threshold", "10")
+
+    @pyqtSlot()
+    def on_action_100_noise_threshold_triggered(self):
+        settings.write("default_noise_threshold", "100")
